######################################################################
# diff experiments

from string import join, split, atoi
from DocumentTemplate.DT_Util import html_quote
from struct import pack, unpack
from OFS.History import historicalRevision
#from OFS.History import html_diff
from OFS import ndiff

class ZWikiDiffMixin:
    """
    This mix-in class adds some diff methods to ZWikiPage.
    """
    
    def lasttext(self):
        """return text of last revision"""
        # get zodb serial no. of last revision
        lastrevision = self.manage_change_history()[1]
        key = lastrevision['key']
        serial=apply(pack, ('>HHHH',)+tuple(map(atoi, split(key,'.'))))

        lastself=historicalRevision(self, serial)
        return lastself.text()

    def diff(self):
        """display a diff with last revision, using ndiff"""
        #diff=html_diff(self.text(),self.lasttext())

        a=split(self.lasttext(),'\n')
        b=split(self.text(),'\n')
        cruncher=ndiff.SequenceMatcher(isjunk=split, a=a, b=b)
        
        r=['<table border=1 width="100%">']
        for tag, alo, ahi, blo, bhi in cruncher.get_opcodes():
            if tag == 'replace':
                replace(a, alo, ahi, b, blo, bhi, r)
            elif tag == 'delete':
                dump('-', a, alo, ahi, r)
            elif tag == 'insert':
                dump('+', b, blo, bhi, r)
            elif tag == 'equal':
                pass     #dump(' ', a, alo, ahi, r)
            else:
                raise ValueError, 'unknown tag ' + `tag`
        r.append('</table>')
        diff = join(r, '\n')
        return '<html>\n<body>\n'+diff+'</body>\n</html>\n'

# output routines for diff
def dump(tag, x, lo, hi, r):
    r1=[]
    r2=[]
    for i in xrange(lo, hi):
        r1.append(tag)
        r2.append(x[i])
    r.append("<tr>\n"
            "<td valign=top width=1%%><pre>\n%s\n</pre></td>\n"
            "<td valign=top width=99%%><pre>\n%s\n</pre></td>\n"
            "</tr>\n"
            % (join(r1,'\n'), html_quote(join(r2, '\n'))))

def replace(x, xlo, xhi, y, ylo, yhi, r):

    rx1=[]
    rx2=[]
    for i in xrange(xlo, xhi):
        rx1.append('-')
        rx2.append(x[i])

    ry1=[]
    ry2=[]
    for i in xrange(ylo, yhi):
        ry1.append('+')
        ry2.append(y[i])


    r.append("<tr>\n"
            "<td valign=top width=1%%><pre>\n%s\n%s\n</pre></td>\n"
            "<td valign=top width=99%%><pre>\n%s\n%s\n</pre></td>\n"
            "</tr>\n"
            % (join(rx1, '\n'), join(ry1, '\n'),
               html_quote(join(rx2, '\n')), html_quote(join(ry2, '\n'))))


