"""
doctest support

Doctests are represented as python snippets within docstrings, and
appear throughout the zwiki code.  See doctest.py for more info.
Each of the tests below invokes doctest on a module and passes if all
the doctests pass.

notes:

- we need to use a hacked version of doctest to see the ZWikiPage
class, because it's based on an extension class.

- the zope context is available in doctests as "zc"

- exceptions in doctests don't work because zunit catches them first

- when reporting the number of tests, doctest counts contiguous
non-blank >>> lines as one test


old todos:

  I suspect we're running the doctests twice, an undesirable slowdown

  it would be nice to make zunit count each doctest separately

  get zunit to leave exceptions for doctest ?

  get zunit to display line of code like doctest, instead of having to
  define assertion strings ?

  get doctest to print line numbers in such a way that emacs can jump there

"""

from Products.ZWiki.tests import simpletestrunner
import unittest
#import Zope
#from Products.ZWiki.ZWikiPage import *
from cStringIO import StringIO
import sys
from Products.ZWiki.tests import doctest

class DocTests(simpletestrunner.TestCase):

    def setUp(self):
        # aborting the transaction breaks the DeleteMe tests
        # for some reason
        #get_transaction().begin()

        # create a dummy page for doctests to play with
        self.ZopeContext.manage_addProduct['ZWiki'].manage_addZWikiPage(
            'TestPage', file='testing testing 1 2 3')

    def tearDown(self):
        self.ZopeContext.manage_delObjects(['TestPage'])
        # XXX shouldn't be necessary if we abort ?

        #del self.ZopeContext   
        #get_transaction().abort()

    def run_doctest(self, module, verbose=0, report=0):
        """
        helper method to invoke doctest on a module from within a
        zunit test
        """
        doctest.master = None   # to get rid of merge warnings
        saveout=sys.stdout
        sys.stdout=StringIO()  # XXX some streaming here perhaps
        failures, tests = doctest.testmod(module,
                                          globs={'zc':self.ZopeContext}, 
                                          verbose=verbose,
                                          report=report)
        sys.stdout.flush()
        output = sys.stdout.getvalue()
        sys.stdout=saveout
        assert failures == 0, 'module %s (%d doctests):\n%s' % \
               (module.__name__,tests,output)
        return

    # run any doctests in the main modules

    def test_init_doctests(self):
        "__init.py__ doctests"
        from Products.ZWiki import __init__; self.run_doctest(__init__)

    # this runs the ZWikiPage doctests twice
    def test_ZWikiAdd_doctests(self):
        "ZWikiAdd.py doctests"
        from Products.ZWiki import ZWikiAdd; self.run_doctest(ZWikiAdd)

    def test_ZWikiDefaults_doctests(self):
        "ZWikiDefaults.py doctests"
        from Products.ZWiki import ZWikiDefaults; self.run_doctest(ZWikiDefaults)

    def test_ZWikiDiff_doctests(self):
        "ZWikiDiff.py doctests"
        from Products.ZWiki import ZWikiDiff; self.run_doctest(ZWikiDiff)

    def test_ZWikiPage_doctests(self):
        "ZWikiPage.py doctests"
        from Products.ZWiki import ZWikiPage; self.run_doctest(ZWikiPage)

    def test_ZWikiParents_doctests(self):
        "ZWikiParents.py doctests"
        from Products.ZWiki import ZWikiParents; self.run_doctest(ZWikiParents)

    def test_ZWikiPermissions_doctests(self):
        "ZWikiPermissions.py doctests"
        from Products.ZWiki import ZWikiPermissions; self.run_doctest(ZWikiPermissions)

    def test_ZWikiRegexes_doctests(self):
        "ZWikiRegexes.py doctests"
        from Products.ZWiki import ZWikiRegexes; self.run_doctest(ZWikiRegexes)

    def test_wwml_doctests(self):
        "wwml.py doctests"
        from Products.ZWiki import wwml; self.run_doctest(wwml)

    # might we want to keep a bunch of doctests down here in the tests
    # package also ?
    #
    #def test_DocTests_doctests(self):
    #    "tests/DocTests.py doctests"
    #    from Products.ZWiki import DocTests; self.run_doctest(DocTests)
    #
    # would rather scan for modules automatically 



def test_suite():
    suite = simpletestrunner.TestSuite()
    suite.addTest(simpletestrunner.makeSuite(DocTests))
    return suite

def run():
    unittest.TextTestRunner().run(test_suite())

if __name__ == '__main__':
    run()
