import simpletestrunner
import unittest
#import Zope
from Products.ZWiki.ZWikiPage import *

class ZWikiWebsTests(simpletestrunner.TestCase):

    DEFAULT_PAGES = (
        ('basic',
         (('FrontPage',           'structuredtext'),
          ('HelpPage',            'structuredtext'),
          ('JumpSearch',          'htmldtml'),
          ('RecentChanges',       'htmldtml'),
          ('RemoteWikiURL',       'structuredtext'),
          ('SearchPage',          'htmldtml'),
          ('StructuredTextRules', 'structuredtext'),
          ('TextFormattingRules', 'structuredtext'),
          ('UserOptions',         'structuredtextdtml'),
          ('ZWiki',               'structuredtext'),
          )),
        ('classic',
         (('FrontPage',           'classicwiki'),
          ('HelpPage',            'classicwiki'),
          ('RecentChanges',       'htmldtml'),
          ('SearchPage',          'htmldtml'),
          ('UserOptions',         'structuredtextdtml'),
          ('ZWiki',               'classicwiki'),
          )),
        ('zwikidotorg',
         (('AnnoyingQuote',       'structuredtext'),
          ('BookMarks',           'structuredtext'),
          ('FrontPage',           'structuredtext'),
          ('HelpPage',            'structuredtext'),
          ('HierarchalStructure', 'structuredtext'),
          ('JumpSearch',          'htmldtml'),
          ('RecentChanges',       'htmldtml'),
          ('RemoteWikiLinks',     'structuredtext'),
          ('RemoteWikiURL',       'structuredtext'),
          ('SearchPage',          'htmldtml'),
          ('StructuredTextRules', 'structuredtext'),
          ('TextFormattingRules', 'structuredtext'),
          ('TimeZone',            'structuredtext'),
          ('UserName',            'structuredtext'),
          ('UserOptions',         'structuredtextdtml'),
          ('WikiName',            'structuredtext'),
          ('WikiWikiWeb',         'structuredtext'),
          ('ZWiki',               'structuredtext'),
          ('ZopeDotOrg',          'structuredtext'),
          )),
        )

    def setUp( self ):
        get_transaction().begin()
#        if not hasattr(self,'ZopeContext') or not self.ZopeContext:
#            self.ZopeContext = Zope.app()
#            self.used_app = 1

    def tearDown( self ):
#        if hasattr(self,'used_app'):
#            # from Zope.__init__, needed to prevent app() hanging
#            self.ZopeContext._p_jar.close()
#            del self.used_app # are these dels needed ?
#
        del self.ZopeContext   
        get_transaction().abort()


    def test_ZWikiWebs_add_form(self):
        "test the 'add zwiki web' form"

        # the old zunit test simulated a user coming in through the web
        # this should work now
        zc = self.ZopeContext
        zc.manage_addProduct['ZWikiWebs'].ZWikiWebAddForm(client=zc,\
                                                          REQUEST=zc.REQUEST)

        # this one assumes direct zodb manipulation, bypassing zpublisher
        # is it possible/meaningful to test the form in this case ?
        # fake REQUEST because the form expects it
        #req = {}
        #wikitype = 'basic'
        #req['new_id'] = wikitype
        #req['new_title'] = wikitype + ' wiki'
        #req['wiki_type'] = wikitype
        ## req['SERVER_URL'] required
        #self.root.manage_addProduct['ZWikiWebs'].ZWikiWebAddForm(
        #    client=self.root,\
        #    REQUEST=req)

    def test_ZWikiWebs_sample_wikis(self):
        """test the sample zwikiwebs."""

        # how would this look as a doctest ? as follows:
        #(would it give us a global report of all errors more easily ? no)
        #
        #>>> DEFAULT_PAGES = (
        #...     ('basic', 
        #...     (('JumpSearch','htmldtml'),
        #...      ('SearchPage','spam'),
        #...      )),
        #...     )
        #>>> product=zc.manage_addProduct['ZWikiWebs']
        #>>> for wikitype, pages in DEFAULT_PAGES:
        #...     zc.REQUEST['new_id'] = wikitype
        #...     zc.REQUEST['new_title'] = wikitype + ' wiki'
        #...     zc.REQUEST['wiki_type'] = wikitype
        #...     product.ZWikiWebAdd(client=product,REQUEST=zc.REQUEST)
        #...     assert hasattr(zc, wikitype), \
        #               wikitype+" wiki was not created"
        #...     for page, type in pages:
        #...         assert hasattr(zc[wikitype],page), \
        #                   wikitype+"/"+page+" does not exist"
        #...         assert zc[wikitype][page].page_type == type, \
        #                   wikitype+"/"+page+"'s type is not "+type
        #...
        
        zc = self.ZopeContext

        # for each sample wikiweb,
        for wikitype, pages in self.DEFAULT_PAGES:

            # create one
            # fake form input
            zc.REQUEST['new_id'] = wikitype
            zc.REQUEST['new_title'] = wikitype + ' wiki'
            zc.REQUEST['wiki_type'] = wikitype
            product = zc.manage_addProduct['ZWikiWebs']
            product.ZWikiWebAdd(client=product, REQUEST=zc.REQUEST)
            
            # verify that it exists and that all pages listed above
            # are present and correct
            assert hasattr(zc, wikitype), \
                   wikitype+" wiki was not created"
            for page, type in pages:
                assert hasattr(zc[wikitype],page), \
                       wikitype+"/"+page+" does not exist"
                assert zc[wikitype][page].page_type == type, \
                       wikitype+"/"+page+"'s type is not "+type


            # also:
            # XXX check dtml methods

            # XXX check for extraneous pages/objects not listed above

            # XXX check other stuff on ZWikiWebs checklist



def test_suite():
    suite = simpletestrunner.TestSuite()
    suite.addTest(simpletestrunner.makeSuite(ZWikiWebsTests))
    return suite

def run():
    unittest.TextTestRunner().run(test_suite())

if __name__ == '__main__':
    run()
